@@ -76,6 +76,7 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
     self.remote_radio_sound_port = self.remote_ctl_base_port + 2
     self.thread_lock = threading.Lock()
     self.remote_radio_ip = socket.gethostbyname(self.conf.remote_radio_ip)	# Allow either host name or IP address
+    self.first_heartbeat = True
 
     self.cw_keydown = 0
     self.cw_phrase_begin_ts = None	# timestamp of beginning of cw phrase
@@ -129,6 +130,7 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
     self.remote_ctl_connected = False
     QS.stop_control_head_remote_sound()
     self.app.main_frame.SetConfigText("Disconnected from remote radio " + self.conf.remote_radio_ip)
+    self.first_heartbeat = True
 
   def RemoteCtlConnect(self):
     if self.remote_ctl_connected:
@@ -136,12 +138,44 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
     if not self.remote_ctl_socket:
       self.remote_ctl_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
       self.remote_ctl_socket.setsockopt(socket.IPPROTO_IP, socket.IP_TOS, 184)	# DSCP "Expedite" (46)
-      self.remote_ctl_socket.settimeout(1.0)	# Allow some time for connection response
+      if DEBUG: print("Default timeout for remote_ctl_socket = ", self.remote_ctl_socket.gettimeout())
+      self.remote_ctl_socket.settimeout(0.1)	# Allow some time for connection response
+      if DEBUG: print("Our set timeout for remote_ctl_socket = ", self.remote_ctl_socket.gettimeout())
     try:
       self.remote_ctl_socket.connect((self.remote_radio_ip, self.remote_ctl_base_port))
     except OSError as err:
-      if DEBUG: print("Remote Control socket.connect() error: {0}".format(err))
-      return False      # Failure to connect
+      if str(err).startswith('[WinError 10056]'):
+        # Connected, in spite of "error" --
+        # This can occur in Windows; Windows networking infrastructure continues attempting to connect,
+        #   even after the connect() call times out (as set in settimeout(), above).
+        #   If Windows connects after the timeout, then, the next time we call connect(),
+        #   it returns WinError 10056, "A connect request was made on an already connected socket".
+        #   Let's accept this "error" as success!
+        if DEBUG: print('connect() returned WinError 10056; Windows already connected! Good!')
+        pass
+      else:
+        # Not yet connected; errors may be expected/normal, or unexpected.
+        if str(err).startswith('time'):
+          # Timeout "error" is normal when we are waiting for Remote Radio server to become available.
+          if DEBUG: print('connect() returned timeout; still waiting for remote radio server')
+        elif str(err).startswith('[WinError 10022]'):
+          # This can occur in Windows; Windows networking infrastructure continues attempting to connect,
+          #   even after the connect() call times out (as set in settimeout(), above).
+          #   If, by the next time we call connect(), it has not yet connected, but is still trying to do so
+          #   as a continuation of the prior call to connect(), it returns WinError 10022,
+          #   "An invalid argument was supplied", which is a little misleading, but okay for us.
+          #   We have not yet connected, but this is an "expected behavior".
+          if DEBUG: print('connect() returned WinError 10022; invalid argument; still waiting for remote radio server')
+        elif str(err).startswith('[Errno 103]'):
+          # This can occur in Linux; still attempting to connect
+          if DEBUG: print('connect() returned Errno 103; software connection abort; still waiting for remote radio server')
+        elif str(err).startswith('[Errno 111]'):
+          # This can occur in Linux; still attempting to connect
+          if DEBUG: print('connect() returned Errno 111; connection refused; still waiting for remote radio server')
+        else:
+          # Unexpected error.  Print error info, regardless of DEBUG status.
+          print("Remote Control socket.connect() error: {0}".format(err))
+        return False      # Failure to connect
     self.remote_ctl_connected = True
     self.remote_ctl_socket.settimeout(0.0)	# Now that we're connected, don't wait if nothing there
     if DEBUG: print("Remote Control connected")
@@ -195,6 +229,16 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
         self.cw_test_next_msec = 0		# Start phrase at 0 msec
         self.num_dits_cur_count = 0
 
+  def ThreadPrinter(self, *args, **kw):
+    # Call this to print from (possibly) the sound thread, which must not be slowed down by a print.
+    # example:  print (x, y, end=' ') ---->  self.ThreadPrinter(x, y, end=' ')
+    with self.thread_lock:	# Call thread_lock only once; twice will deadlock.
+      # If print request is from within sound thread, pass the print request to the GUI thread.
+      if threading.current_thread().name == "QuiskSound":
+        wx.CallAfter(print, *args, **kw)
+      else:
+        print(*args, **kw)
+
   def PollCwKey(self):		# Called by the sound thread
     if DEBUG_CW_SEND_DITS:
       self.SendCwDits()
@@ -215,7 +259,7 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
       elif key_down == 1:
         # Quisk startup with key down
         t = f'Quisk is starting with CW key down!  Tx is on, and Rx is blocked until you release CW key.'
-        print(t)
+        self.ThreadPrinter(t)
         #dlg = wx.MessageDialog(self.app.main_frame, t, "Quisk start, CW key down", style=wx.OK)
         #wx.CallAfter(dlg.ShowModal)
         self.cw_phrase_begin_ts = ts
@@ -229,13 +273,13 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
       t = f'CW;{key_down};{cw_event_ts_msecs}\n'
       self.RemoteCtlSend(t)
       self.cw_phrase_end_ts = ts # End-of-cw-phrase-detection
-      if DEBUG_CW_JITTER: print(f'{ts:10.4f} {key_down}, {cw_event_ts_msecs}')
+      if DEBUG_CW_JITTER: self.ThreadPrinter(f'{ts:10.4f} {key_down}, {cw_event_ts_msecs}')
 
   def HeartBeat(self):	# Called at about 10 Hz by the main
     if self.closing:	# Don't try to connect if we are closing
       return
     ts = time.time()
-    if (ts - self.remote_ctl_timestamp) > 1.0:
+    if (ts - self.remote_ctl_timestamp) > 1.0 or self.first_heartbeat:
       self.remote_ctl_timestamp = ts
       if self.remote_ctl_connected:
         # Send keep-alive heartbeat command
@@ -245,21 +289,26 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
         # Else continually try to connect
         if DEBUG: print('Heartbeat Connect Attempt')
         self.RemoteCtlConnect()
+        self.first_heartbeat = False
     self.RemoteCtlRead()
 
   def RemoteCtlSend(self, text):
-    with self.thread_lock:
-      if not self.remote_ctl_connected:
-        if DEBUG: print('Cannot send if not TCP connected:', text)
-        return
-      #if text[0:4] not in ("FREQ", "HEAR"):
-      #  print ("Control head RemoteCtlSend", text)
-      #BMC if DEBUG: print('Send', text, end=' ')
-      try:	# Patch thanks to Rolandas, LY0NAS
+    # RemoteCtlSend() may be called from sound thread or GUI thread!
+    # self.thread lock (also used in self.ThreadPrinter) protects against thread collisions.
+    if not self.remote_ctl_connected:
+      if DEBUG: self.ThreadPrinter('Cannot send if not TCP connected:', text)
+      return
+    if DEBUG: self.ThreadPrinter('Send: ', text, end=' ')
+    with self.thread_lock: # Do not call ThreadPrinter() from another thread lock!
+      try:
         self.remote_ctl_socket.sendall(text.encode('utf-8', errors='ignore'))
-      except socket.error:
-        if DEBUG: print('Closing remote control socket: error in RemoteCtlSend')
-        self.RemoteCtlClose()
+      except OSError as err:
+        errtxt = err
+        pass
+      else:
+        return
+    self.ThreadPrinter("Closing remote control socket; error in RemoteCtlSend(): {0}".format(errtxt))
+    self.RemoteCtlClose()
 
   def GetSmeter(self):
     return self.smeter_text
@@ -278,8 +327,9 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
     while '\n' in self.received:	# A complete response ending with newline is available
       reply, self.received = self.received.split('\n', 1)	# Split off the reply, save any further characters
       reply = reply.strip()		# Here is our reply
+      if DEBUG: print('Rcvd: ', reply)
       if reply[0] in 'Qq':
-        print('Closing Remote Control socket: Q from remote radio')
+        print('Closing Remote Control socket: Q (Quit) from remote radio')
         self.RemoteCtlClose()
         return
       elif reply[0] in 'Mm':
@@ -320,7 +370,8 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
     self.RemoteCtlSend("%s;1\n" % app.lastBand)	
     # Mode
     btn = app.modeButns.GetSelectedButton()
-    self.RemoteCtlSend("%s;%d\n" % (btn.idName, btn.GetIndex()))
+    if btn:
+      self.RemoteCtlSend("%s;%d\n" % (btn.idName, btn.GetIndex()))
     # Filter and adjustable bandwidth
     name = "Filter 6Slider"
     value = app.midiControls[name][0].button.slider_value
