@@ -249,8 +249,8 @@ struct QuiskWav {			// data to create a WAV or RAW audio file
 // Remote Quisk control head and slave by Ben, AC2YD
 extern int remote_control_head;
 extern int remote_control_slave;
-int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamples, int nSamples);
-int read_remote_mic_sound_socket(struct sound_dev* dev, complex double * cSamples);
+int read_remote_radio_sound_socket(complex double * cSamples);
+int read_remote_mic_sound_socket(complex double * cSamples);
 void send_remote_radio_sound_socket(complex double * cSamples, int nSamples);
 void send_remote_mic_sound_socket(complex double * cSamples, int nSamples);
 extern PyObject * quisk_start_control_head_remote_sound(PyObject * self, PyObject * args);
@@ -290,9 +290,8 @@ extern int quiskImdLevel;				// level for rxMode IMD
 extern int quiskTxHoldState;			// state machine for Tx wait for repeater frequency shift
 extern double quisk_ctcss_freq;			// frequency in Hertz
 extern unsigned char quisk_pc_to_hermes[17 * 4];		// Data to send from the PC to the Hermes hardware
-extern unsigned char quisk_hermeslite_writequeue[4 * 5];	// One-time writes to Hermes-Lite
-extern unsigned int quisk_hermeslite_writepointer;		// write pointer into write queue, nonzero value triggers writes, 
-extern unsigned int quisk_hermeslite_writeattempts;		// counter for write retries
+extern unsigned char quisk_hermeslite_writequeue[5];		// One-time writes to Hermes-Lite
+extern unsigned int quisk_hermeslite_writepointer;		// 0==No request; 1=Send writequeue; 2==Wait for ACK; 3==0x3F error from HL2
 extern unsigned int quisk_hermes_code_version;			// Hermes code version from Hermes to PC
 extern unsigned int quisk_hermes_board_id;			// Hermes board ID from Hermes to PC
 extern int hermes_mox_bit;					// Hermes mox bit from the PC to Hermes
@@ -316,6 +315,7 @@ extern play_state_t quisk_play_state;		// startup, receiving, sidetone
 extern int freedv_current_mode;			// current FreeDV mode; 700D, @)@) etc.
 extern int n_modem_sample_rate;			// Receive data, decimate to modem_sample_rate, FreeDV codec output data at speech_sample_rate
 extern int n_speech_sample_rate;		// Microphone decimate to speech_sample_rate, Freedv codec output data at modem_sample_rate, interpolate to 48000
+extern int n_max_modem_samples;			// maximum input to freedv_rx()
 extern int quisk_start_cw_delay;		// milliseconds to delay output on serial or MIDI CW key down
 extern int quisk_start_ssb_delay;		// milliseconds to discard output for all modes except CW
 extern struct sound_dev * quiskPlaybackDevices[];	// array of Playback sound devices
@@ -405,8 +405,8 @@ void quisk_set_play_state(void);
 void quisk_poll_hardware_key(void);
 
 // Functions supporting digital voice codecs
-typedef int  (* ty_dvoice_codec_rx)(complex double *, double *, int, int);
-typedef int  (* ty_dvoice_codec_tx)(complex double *, double *, int);
+typedef int  (* ty_dvoice_codec_rx)(short *, double *, int, int);
+typedef int  (* ty_dvoice_codec_tx)(complex double *, double *, int, int);
 extern ty_dvoice_codec_rx  pt_quisk_freedv_rx;
 extern ty_dvoice_codec_tx  pt_quisk_freedv_tx;
 
@@ -419,6 +419,7 @@ void quisk_close_sound_alsa(struct sound_dev **, struct sound_dev **);
 
 int  quisk_read_portaudio(struct sound_dev *, complex double *);
 void quisk_play_portaudio(struct sound_dev *, int, complex double *, int, double);
+void quisk_pulseaudio_sidetone(struct sound_dev *);
 void quisk_start_sound_portaudio(struct sound_dev **, struct sound_dev **);
 void quisk_close_sound_portaudio(void);
 
