@@ -150,7 +150,7 @@ class Hardware(BaseHardware):
               self.quisk_mode = mode
               if mode in ('CW', 'CWR'):
                 mode = 'CWU'
-              self.application.OnBtnMode(None, mode)		# Set mode
+              self.application.modeButns.SetLabel(mode, True)		# Set mode
         else:
           if DEBUG: print('Unknown', reply)
       except:
