@@ -289,6 +289,8 @@ class SliderBoxH:
       if application.remote_control_head:
         application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetValue()}\n')
       self.handler(None)
+  def SetFocus(self):
+    self.slider.SetFocus()
 
 class SliderBoxHH(SliderBoxH, wx.BoxSizer):
   """A horizontal control with a slider and text with a value.  The text must have a %d if display is True."""
@@ -797,16 +799,19 @@ class QFilterButtonWindow(wx.Frame):
     x, y = wrap.GetPosition().Get()
     x, y = wrap.GetParent().ClientToScreen(wx.Point(x, y))
     w, h = wrap.GetSize()
-    height = h * 10
+    height = h * 12
     size = (w, height)
     if sys.platform == 'win32':
-      pos = (x, y - height)
+      pos = (x, y - height - h)
       t = 'Filter'
     else:
       pos = (x, y - height - h)
       t = ''
     wx.Frame.__init__(self, wrap.GetParent(), -1, t, pos, size,
       wx.FRAME_TOOL_WINDOW|wx.FRAME_FLOAT_ON_PARENT|wx.CLOSE_BOX|wx.CAPTION|wx.SYSTEM_MENU)
+    self.SetSizeHints(w, height, w, height)
+    hbox = wx.BoxSizer(wx.VERTICAL)
+    self.SetSizer(hbox)
     self.SetBackgroundColour(conf.color_freq)
     self.Bind(wx.EVT_CLOSE, self.OnClose)
     try:
@@ -814,19 +819,18 @@ class QFilterButtonWindow(wx.Frame):
     except ValueError:
       index = 0
       self.wrap.button.slider_value = self.valuelist[0]
-    self.slider = wx.Slider(self, -1, index, 0, 100, (0, 0), (w//2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
+    self.slider = wx.Slider(self, -1, index, 0, 100, style=wx.SL_VERTICAL|wx.SL_INVERSE)
+    hbox.Add(self.slider, flag=wx.CENTER, proportion=1)
     self.slider.Bind(wx.EVT_SCROLL, self.OnSlider)
     self.SetTitle("%d" % self.valuelist[index])
     self.Show()
     self.slider.SetFocus()
+    self.Fit()
   def OnSlider(self, event):
     index = self.slider.GetValue()
     value = self.valuelist[index]
     self.SetTitle("%d" % value)
     self.wrap.ChangeSlider(value)
-    #self.wrap.SetLabel(str(value))
-    #self.wrap.SetValue(True, True)
-    #application.filterAdjBw1 = value
   def OnClose(self, event):
     self.wrap.adjust = None
     self.Destroy()
@@ -838,25 +842,29 @@ class QSliderButtonWindow(wx.Frame):
     x, y = button.GetPosition().Get()
     x, y = button.GetParent().ClientToScreen(wx.Point(x, y))
     w, h = button.GetSize()
-    height = h * 10
+    height = h * 12
     size = (w, height)
     if sys.platform == 'win32':
-      pos = (x, y - height)
+      pos = (x, y - height - h)
     else:
       pos = (x, y - height - h)
     wx.Frame.__init__(self, button.GetParent(), -1, '', pos, size,
       wx.FRAME_TOOL_WINDOW|wx.FRAME_FLOAT_ON_PARENT|wx.CLOSE_BOX|wx.CAPTION|wx.SYSTEM_MENU)
+    self.SetSizeHints(w, height, w, height)
+    hbox = wx.BoxSizer(wx.VERTICAL)
+    self.SetSizer(hbox)
     self.SetBackgroundColour(conf.color_freq)
     self.Bind(wx.EVT_CLOSE, self.OnClose)
     self.slider = wx.Slider(self, -1, value,
-             self.button.slider_min, self.button.slider_max,
-             (0, 0), (w//2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
+             self.button.slider_min, self.button.slider_max, style=wx.SL_VERTICAL|wx.SL_INVERSE)
+    hbox.Add(self.slider, flag=wx.CENTER, proportion=1)
     self.slider.Bind(wx.EVT_SCROLL, self.OnSlider)
     if self.button.display:
       value = float(value) / self.button.slider_max
       self.SetTitle("%6.3f" % value)
     self.Show()
     self.slider.SetFocus()
+    self.Fit()
   def OnSlider(self, event):
     value = self.slider.GetValue()
     if self.button.display:
@@ -1280,7 +1288,7 @@ class RadioButtonGroup:
       index = 0
     for i in range(length):
       shortcut = self.shortcuts[index]
-      if shortcut and wx.GetKeyState(ord(shortcut)):
+      if shortcut and application.QuiskGetKeyState(ord(shortcut)):
         break
       index += 1
       if index >= length:
@@ -1313,7 +1321,7 @@ class _PopWindow(wx.PopupWindow):
 
 class RadioBtnPopup:
   """This class contains a button that pops up a row of radio buttons"""
-  def __init__(self, parent, command, in_labels, default):
+  def __init__(self, parent, command, in_labels, default, idName):
     self.parent = parent
     self.pop_command = command
     self.button_data = {}
@@ -1325,10 +1333,12 @@ class RadioBtnPopup:
       else:
         labels.append(item)
     self.RbDialog = _PopWindow(parent, self.OnGroupButton, labels, default)
+    self.RbDialog.RbGroup.idName = idName
     self.RbDialog.Hide()
     self.pop_control = wx.BoxSizer(wx.HORIZONTAL)
     self.first_button = QuiskPushbutton(parent, self.OnFirstButton, labels[0], text_color=conf.color_popchoice)
     self.first_button.decoration = u'\u21D2'
+    self.first_button.idName = idName
     self.second_button = QuiskBitmapButton(parent, self.OnSecondButton, _bitmap_menupop, use_right=True)
     self.pop_control.Add(self.first_button, 1, flag=wx.ALIGN_CENTER_VERTICAL)
     self.pop_control.Add(self.second_button, 0, flag=wx.ALIGN_CENTER_VERTICAL|wx.LEFT, border=2)
@@ -1446,6 +1456,8 @@ class RadioBtnPopup:
     return self.first_button.GetSize()
   def Shortcut(self, event, button_name=None):
     self.SetLabel(button_name, True)
+  def GetSelectedButton(self):		# return the selected button
+    return self.RbDialog.RbGroup.GetSelectedButton()
 
 class FreqSetter(wx.TextCtrl):
   def __init__(self, parent, x, y, label, fmin, fmax, freq, command):
@@ -1537,6 +1549,8 @@ class QuiskMenu(wx.Menu):
     item.Check(is_checked)
     self.InitData(item, handler)
     return item
+  def AppendSeparator(self):
+    wx.Menu.AppendSeparator(self)
   def IsItemChecked(self, item_text):
     nid = self.item_text2id[item_text]
     menu_item = self.FindItemById(nid)
