@@ -16,6 +16,14 @@
 #include <sys/socket.h>
 #include <arpa/inet.h>
 #include <netinet/ip.h>
+
+#if defined(__unix__) || (defined(__APPLE__) && defined(__MACH__))
+#include <sys/param.h>
+#if defined(BSD)
+#include <netinet/in.h>
+#endif
+#endif
+
 #endif
 
 #include "quisk.h"
@@ -805,25 +813,27 @@ void * quisk_make_sidetone(struct sound_dev * dev, int rewind)
 }
 
 int quisk_play_sidetone(struct sound_dev * dev)
-{  // return 1 if we played the ALSA sidetone
-#ifdef QUISK_HAVE_ALSA
+{  // return 1 if we played the ALSA or PULSEAUDIO sidetone
 	static int last_play_state = 0;
 
 	if (quisk_play_state <= RECEIVE && last_play_state <= RECEIVE)
 		return 0;
 	last_play_state = quisk_play_state;
-	if (quisk_active_sidetone != 3)
-		return 0;
 	if (quisk_isFDX)
 		return 0;
-	if (dev->driver == DEV_DRIVER_ALSA) {
+#ifdef QUISK_HAVE_ALSA
+	if (quisk_active_sidetone == 3 && dev->driver == DEV_DRIVER_ALSA) {
 		quisk_alsa_sidetone(dev);
 		return 1;
 	}
-	return 0;
-#else
-	return 0;
 #endif
+#ifdef QUISK_HAVE_PULSEAUDIO
+	if (quisk_active_sidetone == 4 && dev->driver == DEV_DRIVER_PULSEAUDIO) {
+		quisk_pulseaudio_sidetone(dev);
+		return 1;
+	}
+#endif
+	return 0;
 }
 
 int quisk_read_sound(void)	// Called from sound thread
@@ -950,7 +960,7 @@ int quisk_read_sound(void)	// Called from sound thread
 #endif
 	// For control_head role in remote control operation, receive radio sound via UDP
 	if (remote_control_head)
-		nSamples = read_remote_radio_sound_socket(&quisk_Playback, cSamples, nSamples);
+		nSamples = read_remote_radio_sound_socket(cSamples);
 	// For remote_radio role in remote control operation, send radio sound via UDP
 	if (remote_control_slave)
 		send_remote_radio_sound_socket(cSamples, nSamples);
@@ -1026,7 +1036,7 @@ int quisk_read_sound(void)	// Called from sound thread
 
 	// For remote_radio role in remote control operation, read mic sound via UDP; replace any mic sound from above.
 	if (remote_control_slave)
-		mic_count = read_remote_mic_sound_socket(&quisk_MicPlayback, cSamples);
+		mic_count = read_remote_mic_sound_socket(cSamples);
 
 	// For control_head role in remote control operation, send mic sound via UDP
 	if (remote_control_head)
