@@ -6,7 +6,7 @@
 #include "quisk.h"
 #include "dsound.h"
 //#include <audiodefs.h>
-#include <Mmreg.h>
+#include <mmreg.h>
 //#include <ksmedia.h>
 //#include <uuids.h>
 #include <stdint.h>
@@ -16,8 +16,6 @@
 
 HRESULT errFound, errOpen;
 
-static int play_buf_bytes;
-
 extern HWND quisk_mainwin_handle;
 
 static GUID IEEE = {0x00000003, 0x0000, 0x0010, {0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}};
@@ -175,8 +173,8 @@ static void MakeWFext(int use_new, int use_float, struct sound_dev * dev, WAVEFO
 	pwfex->Format.nChannels = dev->num_channels;
 	pwfex->Format.nSamplesPerSec = dev->sample_rate;
 	pwfex->Format.nAvgBytesPerSec = dev->num_channels * dev->sample_rate * dev->sample_bytes;
-	play_buf_bytes = pwfex->Format.nAvgBytesPerSec * quisk_sound_state.latency_millisecs / 1000 * 2;
-	dev->play_buf_size = play_buf_bytes / (dev->num_channels * dev->sample_bytes);
+	dev->play_buf_bytes = pwfex->Format.nAvgBytesPerSec * quisk_sound_state.latency_millisecs / 1000 * 2;
+	dev->play_buf_size = dev->play_buf_bytes / (dev->num_channels * dev->sample_bytes);
 	pwfex->Format.nBlockAlign = dev->num_channels * dev->sample_bytes;
 	pwfex->Format.wBitsPerSample = dev->sample_bytes * 8;
 }
@@ -219,7 +217,7 @@ static int quisk_open_capture(struct sound_dev * dev)
 	memset(&dscbd, 0, sizeof(DSCBUFFERDESC));
 	dscbd.dwSize = sizeof(DSCBUFFERDESC);
 	dscbd.dwFlags = 0;
-	dscbd.dwBufferBytes = play_buf_bytes;
+	dscbd.dwBufferBytes = dev->play_buf_bytes;
 	dscbd.lpwfxFormat = (WAVEFORMATEX *)&wfex;
 	hr = IDirectSoundCapture_CreateCaptureBuffer(
 		(LPDIRECTSOUNDCAPTURE8)dev->handle, &dscbd, &ptBuf, NULL);
@@ -245,7 +243,7 @@ static int quisk_open_capture(struct sound_dev * dev)
 			dev->sample_rate, dev->channel_I, dev->channel_Q);
                 QuiskPrintf("  Number of channels %d, bytes per sample %d\n", dev->num_channels, dev->sample_bytes);
 		QuiskPrintf("Created capture buffer size %d bytes for device %s, descr %s\n",
-			play_buf_bytes, dev->name, dev->stream_description);
+			dev->play_buf_bytes, dev->name, dev->stream_description);
 	}
 	return 0;
 }
@@ -292,7 +290,7 @@ static int quisk_open_playback(struct sound_dev * dev)
 	memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));
 	dsbdesc.dwSize = sizeof(DSBUFFERDESC); 
 	dsbdesc.dwFlags = DSBCAPS_GETCURRENTPOSITION2|DSBCAPS_GLOBALFOCUS;
-	dsbdesc.dwBufferBytes = play_buf_bytes;
+	dsbdesc.dwBufferBytes = dev->play_buf_bytes;
 	dsbdesc.lpwfxFormat = (LPWAVEFORMATEX)&wfex;
 	hr = IDirectSound_CreateSoundBuffer(
 		(LPDIRECTSOUND8)dev->handle, &dsbdesc, &ptBuf, NULL); 
@@ -309,7 +307,7 @@ static int quisk_open_playback(struct sound_dev * dev)
 			dev->sample_rate, dev->channel_I, dev->channel_Q);
                 QuiskPrintf("  Number of channels %d, bytes per sample %d\n", dev->num_channels, dev->sample_bytes);
 		QuiskPrintf("Created play buffer size %d bytes for device %s, descr %s\n",
-			play_buf_bytes, dev->name, dev->stream_description);
+			dev->play_buf_bytes, dev->name, dev->stream_description);
 	}
 	return 0;
 }
@@ -405,7 +403,7 @@ int  quisk_read_directx(struct sound_dev * dev, complex double * cSamples)
         if (readPos >= dev->dataPos)
 	        bytes = readPos - dev->dataPos;
         else
-	        bytes = readPos - dev->dataPos + play_buf_bytes;
+	        bytes = readPos - dev->dataPos + dev->play_buf_bytes;
 	frames = bytes / bytes_per_frame;	// frames available to read
 	dev->dev_latency = frames;
 //if (dev->dev_index == t_Capture) QuiskPrintf("Read %d frames\n", frames);
@@ -428,8 +426,8 @@ int  quisk_read_directx(struct sound_dev * dev, complex double * cSamples)
 		QuiskPrintf ("Lock not equal to bytes\n");
 #endif
 	dev->dataPos += bytes;
-        if (dev->dataPos >= play_buf_bytes)
-                dev->dataPos -= play_buf_bytes;
+        if (dev->dataPos >= dev->play_buf_bytes)
+                dev->dataPos -= dev->play_buf_bytes;
 	nSamples = 0;
 	switch (dev->sound_format) {
 	case Int16:
@@ -536,10 +534,10 @@ void quisk_play_directx(struct sound_dev * dev, int nSamples,
 	}
 	unavail = (int)writePos - (int)playPos;   // Must not write to this region
 	if (unavail < 0)
-		unavail += play_buf_bytes;
+		unavail += dev->play_buf_bytes;
         dev->play_delay = dev->dataPos - playPos;       // includes unavail
         if (dev->play_delay < 0)
-                dev->play_delay += play_buf_bytes;
+                dev->play_delay += dev->play_buf_bytes;
 	dev->cr_average_fill += ((double)(dev->play_delay - unavail) / bytes_per_frame + nSamples / 2) / (2 * dev->latency_frames);
 	dev->cr_average_count++;
         // buffer_fill is (the writable play samples plus the current samples) / (2 * latency_frames)
@@ -556,7 +554,7 @@ void quisk_play_directx(struct sound_dev * dev, int nSamples,
 		break;
 	case 1: // Normal run state
 		// Measure the space available to write samples
-		frames = (play_buf_bytes - dev->play_delay - unavail) / bytes_per_frame;
+		frames = (dev->play_buf_bytes - dev->play_delay - unavail) / bytes_per_frame;
 		// Check for underrun
 		n = unavail / bytes_per_frame + dev->latency_frames * 2 / 10 - nSamples;   // minimum frames
 		if (dev->dev_latency < n) {
@@ -654,8 +652,8 @@ void quisk_play_directx(struct sound_dev * dev, int nSamples,
 	}
 	IDirectSoundBuffer8_Unlock(ptBuf, pt1, n1, pt2, n2);
 	dev->dataPos += bytes;	// update data write position
-	if (dev->dataPos >= play_buf_bytes)
-		dev->dataPos -= play_buf_bytes;
+	if (dev->dataPos >= dev->play_buf_bytes)
+		dev->dataPos -= dev->play_buf_bytes;
 	dev->dev_latency = dev->play_delay / bytes_per_frame;		// frames in buffer available to play
 	if (report_latency)			// Report latency for main playback device
 		quisk_sound_state.latencyPlay = dev->dev_latency;
