@@ -67,12 +67,16 @@ static pa_stream *OpenPulseDevices[PA_LIST_SIZE * 2] = {NULL};
 static int have_QuiskDigitalInput;	// Do we need to create this null sink?
 static int have_QuiskDigitalOutput;	// Do we need to create this null sink?
 
+static int underrunPlayback;
+static int tlength_bytes;
+
 /* This callback happens any time a stream changes state. Here, it's primary used to 
  * tell the quisk thread when streams are ready. 
  */
 
 static void stream_state_callback(pa_stream *s, void *userdata) {
     struct sound_dev *dev = userdata;
+    unsigned int S;
     assert(s);
     assert(dev);
     
@@ -101,14 +105,16 @@ static void stream_state_callback(pa_stream *s, void *userdata) {
                        pa_stream_is_suspended(s) ? "" : "not ",
                        pa_stream_get_device_name(s));
 
+		S = dev->num_channels * dev->sample_bytes * dev->sample_rate / 1000;
                 if (!(a = pa_stream_get_buffer_attr(s)))
                     printf("   pa_stream_get_buffer_attr() failed: %s", pa_strerror(pa_context_errno(pa_stream_get_context(s))));
                 else if (!(a->prebuf)) {
-                    printf("   Buffer metrics: maxlength=%u, fragsize=%u\n", a->maxlength, a->fragsize);
+                    printf("   Buffer metrics msec: rate %d, channels %d, sample bytes %d, maxlength=%u, fragsize=%u\n",
+			dev->sample_rate, dev->num_channels, dev->sample_bytes, a->maxlength/S, a->fragsize/S);
                 }
                 else {
-                    printf("   Buffer metrics: maxlength=%u, prebuf=%u, tlength=%u  minreq=%u\n",
-                           a->maxlength, a->prebuf, a->tlength, a->minreq);
+                    printf("   Buffer metrics msec: rate %d, channels %d, sample bytes %d, maxlength=%u, prebuf=%u, tlength=%u  minreq=%u\n",
+			dev->sample_rate, dev->num_channels, dev->sample_bytes, a->maxlength/S, a->prebuf/S, a->tlength/S, a->minreq/S);
                 }
             }
             break;
@@ -137,6 +143,8 @@ static void stream_underflow_callback(pa_stream *s, void *userdata) {
     if (quisk_sound_state.verbose_pulse)
         printf("Stream underrun %s\n", dev->stream_description);
     dev->dev_error++;
+    if (dev->dev_index == t_Playback)
+        underrunPlayback = 1;
 }
 
 //Indicates overflow on passed stream (record)
@@ -164,7 +172,7 @@ static void stream_corked_callback(pa_stream *s, int success, void *userdata) {
     assert(s);
     struct sound_dev *dev = userdata;
     
-    if (s) {
+    if (success) {
         if (quisk_sound_state.verbose_pulse)
             printf("Stream cork/uncork %s success\n", dev->stream_description);
         pa_threaded_mainloop_signal(pa_ml, 0);
@@ -181,7 +189,7 @@ static void stream_flushed_callback(pa_stream *s, int success, void *userdata) {
     assert(s);
     struct sound_dev *dev = userdata;
     
-    if (s) {
+    if (success) {
         printf("Stream flush %s success\n", dev->stream_description);
         pa_threaded_mainloop_signal(pa_ml, 0);
     }
@@ -189,6 +197,7 @@ static void stream_flushed_callback(pa_stream *s, int success, void *userdata) {
         printf("Stream flush %s Failure!\n", dev->stream_description);
         exit(12);
     }
+    pa_threaded_mainloop_signal(pa_ml, 0);
 }
 
 // This is called by the play function when the timing structure is updated.
@@ -209,18 +218,51 @@ static void stream_timing_callback(pa_stream *s, int success, void *userdata) {
     pa_threaded_mainloop_signal(pa_ml, 0);
 }
 
+// This is used to change the radio sound playback buffer attributes so the sidetone is faster
+static void buffer_attr_callback(pa_stream * s, int success, void * userdata) {
+    struct sound_dev * dev = userdata;
+    assert(s);
+
+    if (success) {
+        printf("Change buffer attr %s success\n", dev->stream_description);
+        pa_threaded_mainloop_signal(pa_ml, 0);
+    }
+    else {
+        printf("Change buffer attr %s Failure!\n", dev->stream_description);
+        exit(17);
+    }
+}
+void quisk_set_play_attr(struct sound_dev * dev, const pa_buffer_attr * attr) {
+    pa_stream *s = dev->handle;
+    pa_operation *o;
+    
+    pa_threaded_mainloop_lock(pa_ml);
+    
+    if (!(o = pa_stream_set_buffer_attr	(s, attr, buffer_attr_callback, dev))) {
+        printf("quisk_play_attr(): %s\n", pa_strerror(pa_context_errno(pa_stream_get_context(s))));
+        exit(14);
+    }
+    else {
+        while(pa_operation_get_state(o) == PA_OPERATION_RUNNING)
+            pa_threaded_mainloop_wait(pa_ml);
+        pa_operation_unref(o);
+    }
+    pa_threaded_mainloop_unlock(pa_ml);
+}
+
 
 /* This callback allows us to read in the server side stream information so that we
  * can match stream formats and sizes to what is configured in pulseaudio.
 */
 
 static void server_info_cb(pa_context *c, const pa_server_info *info, void *userdata) {
+    int bytes_per_frame;
     struct sound_dev **pDevices = userdata;
     pa_buffer_attr rec_ba;
     pa_buffer_attr play_ba;
     pa_sample_spec ss;
     pa_sample_spec default_ss;
-    pa_stream_flags_t pb_flags = PA_STREAM_NOFLAGS;
+    pa_stream_flags_t pb_flags;
     pa_stream_flags_t rec_flags = PA_STREAM_ADJUST_LATENCY;
     default_ss = info->sample_spec;
 
@@ -261,13 +303,21 @@ static void server_info_cb(pa_context *c, const pa_server_info *info, void *user
         
         ss.rate = dev->sample_rate;
         ss.channels = dev->num_channels;
+        bytes_per_frame = dev->sample_bytes * dev->num_channels;
 
         rec_ba.maxlength = (uint32_t) -1;
         rec_ba.fragsize = (uint32_t) SAMP_BUFFER_SIZE / 16;  //higher numbers eat cpu on reading monitor streams.
         play_ba.maxlength = (uint32_t) -1;
-        play_ba.prebuf = (uint32_t) (dev->sample_bytes * ss.channels * dev->latency_frames);
-        //play_ba.tlength = (uint32_t) -1;
-        play_ba.tlength = play_ba.prebuf;
+        if (dev->dev_index == t_Playback) {	// radio sound playback uses cork/uncork instead of prebuf
+       	    play_ba.prebuf = 0;
+            tlength_bytes = dev->sample_rate / 1000 * quisk_sound_state.data_poll_usec * bytes_per_frame * 2 / 1000;
+            play_ba.tlength = tlength_bytes;
+        }
+        else {
+       	    play_ba.prebuf = bytes_per_frame * dev->latency_frames;
+       	    play_ba.tlength = play_ba.prebuf;
+        }
+
 
         if (dev->latency_frames == 0)
             play_ba.minreq = (uint32_t) 0; //verify this is sane
@@ -294,6 +344,13 @@ static void server_info_cb(pa_context *c, const pa_server_info *info, void *user
                 printf("pa_stream_new() failed: %s", pa_strerror(pa_context_errno(c)));
                 continue;
             }
+            if (dev->dev_index == t_Playback) {
+                pb_flags = PA_STREAM_START_CORKED;
+                dev->cork_status = 1;
+            }
+            else {
+                pb_flags = PA_STREAM_NOFLAGS;
+            }
             if (pa_stream_connect_playback(s, dev_name, &play_ba, pb_flags, pa_cvolume_set(&cv, ss.channels, volume), NULL) < 0) {
                 printf("pa_stream_connect_playback() failed: %s", pa_strerror(pa_context_errno(c)));
                 continue;
@@ -560,14 +617,16 @@ int quisk_read_pulseaudio(struct sound_dev *dev, complex double *cSamples)
 void quisk_play_pulseaudio(struct sound_dev *dev, int nSamples, complex double *cSamples, 
     int report_latency, double volume) {
     pa_stream *s = dev->handle;
-    int i=0, n=0;
+    static const pa_timing_info * timing = NULL;
+    int i=0, n=0, bufferFrames, bytes_per_frame;
     void *fbuffer;
     size_t fbuffer_bytes = 0;
+    size_t writable;
 
     if( !dev || nSamples <= 0 || dev->pulse_stream_state != PA_STREAM_READY)
         return;
     
-    if (dev->cork_status)
+    if (dev->cork_status && dev->dev_index != t_Playback)
         return;
     
     if (report_latency) {     // Report the latency, if requested.
@@ -583,13 +642,31 @@ void quisk_play_pulseaudio(struct sound_dev *dev, int nSamples, complex double *
                 pa_threaded_mainloop_wait(pa_ml);
             pa_operation_unref(o);
         }
+	if (timing == NULL)
+		timing = pa_stream_get_timing_info(s);
         
         pa_threaded_mainloop_unlock(pa_ml);
         
     }
-    
 
-    fbuffer = pa_xmalloc(nSamples * dev->num_channels * dev->sample_bytes);
+    bytes_per_frame = dev->sample_bytes * dev->num_channels;
+    if (dev->dev_index == t_Playback) {
+    	bufferFrames = (timing->write_index - timing->read_index) / bytes_per_frame;
+        if (quisk_sound_state.verbose_pulse > 2)
+            printf("read %ld, write %ld, frames %d\n", timing->read_index, timing->write_index, bufferFrames);
+        if (dev->cork_status) {
+            if (bufferFrames >= dev->latency_frames)
+              quisk_cork_pulseaudio(dev, 0);
+        }
+        else {
+            if (underrunPlayback) {
+                underrunPlayback = 0;
+                quisk_cork_pulseaudio(dev, 1);
+            }
+        }
+    }
+
+    fbuffer = pa_xmalloc(nSamples * bytes_per_frame);
    
     // Convert from complex data to framebuffer
 
@@ -620,9 +697,12 @@ void quisk_play_pulseaudio(struct sound_dev *dev, int nSamples, complex double *
 
 
 
-    fbuffer_bytes = nSamples * dev->num_channels * dev->sample_bytes;
+    fbuffer_bytes = nSamples * bytes_per_frame;
     pa_threaded_mainloop_lock(pa_ml);
-    size_t writable = pa_stream_writable_size(s);
+    if (dev->dev_index == t_Playback)	// defeat pa_stream_writable_size()
+        writable = 1024 * 1000;
+    else
+        writable = pa_stream_writable_size(s);
 
     if (writable > 0) {
         if ( writable > 1024*1000 ) //sanity check to prevent pa_xmalloc from crashing on monitor streams
@@ -641,9 +721,102 @@ void quisk_play_pulseaudio(struct sound_dev *dev, int nSamples, complex double *
     
     pa_threaded_mainloop_unlock(pa_ml);
     pa_xfree(fbuffer);
-    fbuffer=NULL;
 }
 
+void quisk_pulseaudio_sidetone(struct sound_dev * dev)
+{
+	int i, ch_I, ch_Q, flags, target_frames, fbuffer_bytes, bytes_per_sample, bytes_per_frame, bufferFrames, writeFrames;
+	static play_state_t old_play_state = RECEIVE;
+	static const pa_timing_info * timing = NULL;
+	pa_operation * o;
+	pa_stream * s = dev->handle;	
+	void * fbuffer, * buffer;
+	void * ptSample;
+
+	if( !dev || dev->pulse_stream_state != PA_STREAM_READY)
+		return;
+
+	bytes_per_sample = dev->sample_bytes;
+	bytes_per_frame = dev->sample_bytes * dev->num_channels;
+	pa_threaded_mainloop_lock(pa_ml);
+	if (!(o = pa_stream_update_timing_info(s, stream_timing_callback, dev))) {
+		printf("pa_stream_update_timing(): %s\n", pa_strerror(pa_context_errno(pa_stream_get_context(s))));
+	}
+	else {
+		while (pa_operation_get_state(o) == PA_OPERATION_RUNNING)
+			pa_threaded_mainloop_wait(pa_ml);
+		pa_operation_unref(o);
+	}
+	if (timing == NULL)
+		timing = pa_stream_get_timing_info(s);
+	pa_threaded_mainloop_unlock(pa_ml);
+
+	if (timing->read_index_corrupt || timing->write_index_corrupt) {
+		printf ("Index corrupt in quisk_pulseaudio_sidetone()\n");
+		return;
+	}
+
+	bufferFrames = (timing->write_index - timing->read_index) / bytes_per_frame;
+       	if (quisk_sound_state.verbose_pulse > 2)
+		printf("read %ld, write %ld, frames %d\n", timing->read_index, timing->write_index, bufferFrames);
+
+	if (old_play_state <= RECEIVE && quisk_play_state > RECEIVE) {		// change from Rx to Tx
+        	if (quisk_sound_state.verbose_pulse > 2)
+			printf("\nChange to Tx\n");
+		old_play_state = quisk_play_state;
+	}
+	else if (old_play_state > RECEIVE && quisk_play_state <= RECEIVE) {	// change from Tx to Rx
+        	if (quisk_sound_state.verbose_pulse > 2)
+			printf("\nChange to Rx\n");
+		dev->old_key = 0;
+		old_play_state = quisk_play_state;
+        	writeFrames = dev->latency_frames;
+		fbuffer_bytes = writeFrames * bytes_per_frame;
+		fbuffer = pa_xmalloc(fbuffer_bytes);
+		memset(fbuffer, 0, fbuffer_bytes);
+		pa_threaded_mainloop_lock(pa_ml);
+        	pa_stream_write(s, fbuffer, fbuffer_bytes, NULL, 0, PA_SEEK_RELATIVE_ON_READ);
+		pa_threaded_mainloop_unlock(pa_ml);
+		pa_xfree(fbuffer);
+		return;
+	}
+
+	target_frames = tlength_bytes / bytes_per_frame * 4;
+	if (QUISK_CWKEY_DOWN != dev->old_key) {	// key changed, re-write buffer
+		dev->old_key = QUISK_CWKEY_DOWN;
+        	writeFrames = target_frames;
+        	if (quisk_sound_state.verbose_pulse > 2)
+			printf ("\nChange key %d, writeFrames %d\n", QUISK_CWKEY_DOWN, writeFrames);
+		quisk_make_sidetone(dev, bufferFrames);		// rewind sidetone samples
+		flags = PA_SEEK_RELATIVE_ON_READ;
+	}
+	else {		// no change to key; keep buffer full
+        	writeFrames = target_frames - bufferFrames;
+        	if (quisk_sound_state.verbose_pulse > 2)
+			printf("writeFrames %d\n", writeFrames);
+		flags = PA_SEEK_RELATIVE;
+	}
+
+	if (writeFrames <= 0)
+		return;
+	fbuffer_bytes = writeFrames * bytes_per_frame;
+	fbuffer = pa_xmalloc(fbuffer_bytes);
+
+	buffer = fbuffer;
+	ch_I = dev->channel_I;
+	ch_Q = dev->channel_Q;
+	for (i = 0; i < writeFrames; i++) {
+		ptSample = quisk_make_sidetone(dev, 0);
+		memcpy(buffer + ch_I * bytes_per_sample, ptSample, bytes_per_sample);
+		memcpy(buffer + ch_Q * bytes_per_sample, ptSample, bytes_per_sample);
+		buffer += bytes_per_frame;
+	}
+
+	pa_threaded_mainloop_lock(pa_ml);
+        pa_stream_write(s, fbuffer, fbuffer_bytes, NULL, 0, flags);
+	pa_threaded_mainloop_unlock(pa_ml);
+	pa_xfree(fbuffer);
+}
 
 //This is a function to sort the device list into local and remote lists.
 static void sort_devices(struct sound_dev **plist, struct sound_dev **pLocal, struct sound_dev **pRemote) {
